@@ -26,6 +26,8 @@ import (
 	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/common/log"
 	"github.com/prometheus/common/version"
+	"golang.org/x/net/context"
+
 	"github.com/prometheus/prometheus/config"
 	"github.com/prometheus/prometheus/notifier"
 	"github.com/prometheus/prometheus/promql"
@@ -73,37 +75,60 @@ func Main() int {
 	log.Infoln("Starting prometheus", version.Info())
 	log.Infoln("Build context", version.BuildContext())
 
-	var reloadables []Reloadable
-
 	var (
-		memStorage     = local.NewMemorySeriesStorage(&cfg.storage)
-		remoteStorage  = remote.New(&cfg.remote)
-		sampleAppender = storage.Fanout{memStorage}
+		sampleAppender = storage.Fanout{}
+		reloadables    []Reloadable
 	)
+
+	var localStorage local.Storage
+	switch cfg.localStorageEngine {
+	case "persisted":
+		localStorage = local.NewMemorySeriesStorage(&cfg.storage)
+		sampleAppender = storage.Fanout{localStorage}
+	case "none":
+		localStorage = &local.NoopStorage{}
+	default:
+		log.Errorf("Invalid local storage engine %q", cfg.localStorageEngine)
+		return 1
+	}
+
+	remoteStorage, err := remote.New(&cfg.remote)
+	if err != nil {
+		log.Errorf("Error initializing remote storage: %s", err)
+		return 1
+	}
 	if remoteStorage != nil {
 		sampleAppender = append(sampleAppender, remoteStorage)
 		reloadables = append(reloadables, remoteStorage)
 	}
 
+	reloadableRemoteStorage := remote.NewConfigurable()
+	sampleAppender = append(sampleAppender, reloadableRemoteStorage)
+	reloadables = append(reloadables, reloadableRemoteStorage)
+
 	var (
-		notifier      = notifier.New(&cfg.notifier)
-		targetManager = retrieval.NewTargetManager(sampleAppender)
-		queryEngine   = promql.NewEngine(memStorage, &cfg.queryEngine)
+		notifier       = notifier.New(&cfg.notifier)
+		targetManager  = retrieval.NewTargetManager(sampleAppender)
+		queryEngine    = promql.NewEngine(localStorage, &cfg.queryEngine)
+		ctx, cancelCtx = context.WithCancel(context.Background())
 	)
 
 	ruleManager := rules.NewManager(&rules.ManagerOptions{
 		SampleAppender: sampleAppender,
 		Notifier:       notifier,
 		QueryEngine:    queryEngine,
+		Context:        ctx,
 		ExternalURL:    cfg.web.ExternalURL,
 	})
 
-	flags := map[string]string{}
-	cfg.fs.VisitAll(func(f *flag.Flag) {
-		flags[f.Name] = f.Value.String()
-	})
+	cfg.web.Context = ctx
+	cfg.web.Storage = localStorage
+	cfg.web.QueryEngine = queryEngine
+	cfg.web.TargetManager = targetManager
+	cfg.web.RuleManager = ruleManager
+	cfg.web.Notifier = notifier
 
-	version := &web.PrometheusVersion{
+	cfg.web.Version = &web.PrometheusVersion{
 		Version:   version.Version,
 		Revision:  version.Revision,
 		Branch:    version.Branch,
@@ -112,11 +137,17 @@ func Main() int {
 		GoVersion: version.GoVersion,
 	}
 
-	webHandler := web.New(memStorage, queryEngine, targetManager, ruleManager, version, flags, &cfg.web)
+	cfg.web.Flags = map[string]string{}
+	cfg.fs.VisitAll(func(f *flag.Flag) {
+		cfg.web.Flags[f.Name] = f.Value.String()
+	})
+
+	webHandler := web.New(&cfg.web)
 
 	reloadables = append(reloadables, targetManager, ruleManager, webHandler, notifier)
 
-	if !reloadConfig(cfg.configFile, reloadables...) {
+	if err := reloadConfig(cfg.configFile, reloadables...); err != nil {
+		log.Errorf("Error loading config: %s", err)
 		return 1
 	}
 
@@ -131,37 +162,48 @@ func Main() int {
 		for {
 			select {
 			case <-hup:
-			case <-webHandler.Reload():
+				if err := reloadConfig(cfg.configFile, reloadables...); err != nil {
+					log.Errorf("Error reloading config: %s", err)
+				}
+			case rc := <-webHandler.Reload():
+				if err := reloadConfig(cfg.configFile, reloadables...); err != nil {
+					log.Errorf("Error reloading config: %s", err)
+					rc <- err
+				} else {
+					rc <- nil
+				}
 			}
-			reloadConfig(cfg.configFile, reloadables...)
 		}
 	}()
 
 	// Start all components. The order is NOT arbitrary.
 
-	if err := memStorage.Start(); err != nil {
+	if err := localStorage.Start(); err != nil {
 		log.Errorln("Error opening memory series storage:", err)
 		return 1
 	}
 	defer func() {
-		if err := memStorage.Stop(); err != nil {
+		if err := localStorage.Stop(); err != nil {
 			log.Errorln("Error stopping storage:", err)
 		}
 	}()
 
 	if remoteStorage != nil {
-		prometheus.MustRegister(remoteStorage)
-
-		go remoteStorage.Run()
+		remoteStorage.Start()
 		defer remoteStorage.Stop()
 	}
+
+	defer reloadableRemoteStorage.Stop()
+
 	// The storage has to be fully initialized before registering.
-	prometheus.MustRegister(memStorage)
+	if instrumentedStorage, ok := localStorage.(prometheus.Collector); ok {
+		prometheus.MustRegister(instrumentedStorage)
+	}
 	prometheus.MustRegister(notifier)
 	prometheus.MustRegister(configSuccess)
 	prometheus.MustRegister(configSuccessTime)
 
-	// The notifieris a dependency of the rule manager. It has to be
+	// The notifier is a dependency of the rule manager. It has to be
 	// started before and torn down afterwards.
 	go notifier.Run()
 	defer notifier.Stop()
@@ -174,7 +216,7 @@ func Main() int {
 
 	// Shutting down the query engine before the rule manager will cause pending queries
 	// to be canceled and ensures a quick shutdown of the rule manager.
-	defer queryEngine.Stop()
+	defer cancelCtx()
 
 	go webHandler.Run()
 
@@ -199,13 +241,13 @@ func Main() int {
 // Reloadable things can change their internal state to match a new config
 // and handle failure gracefully.
 type Reloadable interface {
-	ApplyConfig(*config.Config) bool
+	ApplyConfig(*config.Config) error
 }
 
-func reloadConfig(filename string, rls ...Reloadable) (success bool) {
+func reloadConfig(filename string, rls ...Reloadable) (err error) {
 	log.Infof("Loading configuration file %s", filename)
 	defer func() {
-		if success {
+		if err == nil {
 			configSuccess.Set(1)
 			configSuccessTime.Set(float64(time.Now().Unix()))
 		} else {
@@ -215,13 +257,27 @@ func reloadConfig(filename string, rls ...Reloadable) (success bool) {
 
 	conf, err := config.LoadFile(filename)
 	if err != nil {
-		log.Errorf("Couldn't load configuration (-config.file=%s): %v", filename, err)
-		return false
+		return fmt.Errorf("couldn't load configuration (-config.file=%s): %v", filename, err)
 	}
-	success = true
 
-	for _, rl := range rls {
-		success = success && rl.ApplyConfig(conf)
+	// Add AlertmanagerConfigs for legacy Alertmanager URL flags.
+	for us := range cfg.alertmanagerURLs {
+		acfg, err := parseAlertmanagerURLToConfig(us)
+		if err != nil {
+			return err
+		}
+		conf.AlertingConfig.AlertmanagerConfigs = append(conf.AlertingConfig.AlertmanagerConfigs, acfg)
 	}
-	return success
+
+	failed := false
+	for _, rl := range rls {
+		if err := rl.ApplyConfig(conf); err != nil {
+			log.Error("Failed to apply configuration: ", err)
+			failed = true
+		}
+	}
+	if failed {
+		return fmt.Errorf("one or more errors occurred while applying the new configuration (-config.file=%s)", filename)
+	}
+	return nil
 }
