@@ -135,9 +135,8 @@ type MatrixSelector struct {
 	Offset        time.Duration
 	LabelMatchers metric.LabelMatchers
 
-	// The series iterators are populated at query analysis time.
-	iterators map[model.Fingerprint]local.SeriesIterator
-	metrics   map[model.Fingerprint]metric.Metric
+	// The series iterators are populated at query preparation time.
+	iterators []local.SeriesIterator
 }
 
 // NumberLiteral represents a number.
@@ -169,9 +168,8 @@ type VectorSelector struct {
 	Offset        time.Duration
 	LabelMatchers metric.LabelMatchers
 
-	// The series iterators are populated at query analysis time.
-	iterators map[model.Fingerprint]local.SeriesIterator
-	metrics   map[model.Fingerprint]metric.Metric
+	// The series iterators are populated at query preparation time.
+	iterators []local.SeriesIterator
 }
 
 func (e *AggregateExpr) Type() model.ValueType  { return model.ValVector }
