@@ -21,6 +21,7 @@ import (
 	"sort"
 	"time"
 
+	"github.com/prometheus/client_golang/prometheus"
 	"github.com/prometheus/common/log"
 	"github.com/prometheus/common/model"
 	"golang.org/x/net/context"
@@ -30,6 +31,41 @@ import (
 	"github.com/prometheus/prometheus/util/stats"
 )
 
+const (
+	namespace = "prometheus"
+	subsystem = "engine"
+
+	// The largest SampleValue that can be converted to an int64 without overflow.
+	maxInt64 model.SampleValue = 9223372036854774784
+	// The smallest SampleValue that can be converted to an int64 without underflow.
+	minInt64 model.SampleValue = -9223372036854775808
+)
+
+var (
+	currentQueries = prometheus.NewGauge(prometheus.GaugeOpts{
+		Namespace: namespace,
+		Subsystem: subsystem,
+		Name:      "queries",
+		Help:      "The current number of queries being executed or waiting.",
+	})
+	maxConcurrentQueries = prometheus.NewGauge(prometheus.GaugeOpts{
+		Namespace: namespace,
+		Subsystem: subsystem,
+		Name:      "queries_concurrent_max",
+		Help:      "The max number of concurrent queries.",
+	})
+)
+
+func init() {
+	prometheus.MustRegister(currentQueries)
+	prometheus.MustRegister(maxConcurrentQueries)
+}
+
+// convertibleToInt64 returns true if v does not over-/underflow an int64.
+func convertibleToInt64(v model.SampleValue) bool {
+	return v <= maxInt64 && v >= minInt64
+}
+
 // sampleStream is a stream of Values belonging to an attached COWMetric.
 type sampleStream struct {
 	Metric metric.Metric
@@ -108,7 +144,7 @@ func (r *Result) Matrix() (model.Matrix, error) {
 	}
 	v, ok := r.Value.(model.Matrix)
 	if !ok {
-		return nil, fmt.Errorf("query result is not a matrix")
+		return nil, fmt.Errorf("query result is not a range vector")
 	}
 	return v, nil
 }
@@ -150,7 +186,7 @@ func (e ErrQueryCanceled) Error() string { return fmt.Sprintf("query was cancele
 // it is associated with.
 type Query interface {
 	// Exec processes the query and
-	Exec() *Result
+	Exec(ctx context.Context) *Result
 	// Statement returns the parsed statement of the query.
 	Statement() Statement
 	// Stats returns statistics about the lifetime of the query.
@@ -192,8 +228,8 @@ func (q *query) Cancel() {
 }
 
 // Exec implements the Query interface.
-func (q *query) Exec() *Result {
-	res, err := q.ng.exec(q)
+func (q *query) Exec(ctx context.Context) *Result {
+	res, err := q.ng.exec(ctx, q)
 	return &Result{Err: err, Value: res}
 }
 
@@ -216,32 +252,30 @@ func contextDone(ctx context.Context, env string) error {
 }
 
 // Engine handles the lifetime of queries from beginning to end.
-// It is connected to a storage.
+// It is connected to a querier.
 type Engine struct {
-	// The storage on which the engine operates.
-	storage local.Querier
-
-	// The base context for all queries and its cancellation function.
-	baseCtx       context.Context
-	cancelQueries func()
+	// A Querier constructor against an underlying storage.
+	queryable Queryable
 	// The gate limiting the maximum number of concurrent and waiting queries.
-	gate *queryGate
-
+	gate    *queryGate
 	options *EngineOptions
 }
 
+// Queryable allows opening a storage querier.
+type Queryable interface {
+	Querier() (local.Querier, error)
+}
+
 // NewEngine returns a new engine.
-func NewEngine(storage local.Querier, o *EngineOptions) *Engine {
+func NewEngine(queryable Queryable, o *EngineOptions) *Engine {
 	if o == nil {
 		o = DefaultEngineOptions
 	}
-	ctx, cancel := context.WithCancel(context.Background())
+	maxConcurrentQueries.Set(float64(o.MaxConcurrentQueries))
 	return &Engine{
-		storage:       storage,
-		baseCtx:       ctx,
-		cancelQueries: cancel,
-		gate:          newQueryGate(o.MaxConcurrentQueries),
-		options:       o,
+		queryable: queryable,
+		gate:      newQueryGate(o.MaxConcurrentQueries),
+		options:   o,
 	}
 }
 
@@ -257,11 +291,6 @@ var DefaultEngineOptions = &EngineOptions{
 	Timeout:              2 * time.Minute,
 }
 
-// Stop the engine and cancel all running queries.
-func (ng *Engine) Stop() {
-	ng.cancelQueries()
-}
-
 // NewInstantQuery returns an evaluation query for the given expression at the given time.
 func (ng *Engine) NewInstantQuery(qs string, ts model.Time) (Query, error) {
 	expr, err := ParseExpr(qs)
@@ -282,7 +311,7 @@ func (ng *Engine) NewRangeQuery(qs string, start, end model.Time, interval time.
 		return nil, err
 	}
 	if expr.Type() != model.ValVector && expr.Type() != model.ValScalar {
-		return nil, fmt.Errorf("invalid expression type %q for range query, must be scalar or vector", expr.Type())
+		return nil, fmt.Errorf("invalid expression type %q for range query, must be scalar or instant vector", documentedType(expr.Type()))
 	}
 	qry := ng.newQuery(expr, start, end, interval)
 	qry.q = qs
@@ -309,9 +338,8 @@ func (ng *Engine) newQuery(expr Expr, start, end model.Time, interval time.Durat
 // of an arbitrary function during handling. It is used to test the Engine.
 type testStmt func(context.Context) error
 
-func (testStmt) String() string   { return "test statement" }
-func (testStmt) DotGraph() string { return "test statement" }
-func (testStmt) stmt()            {}
+func (testStmt) String() string { return "test statement" }
+func (testStmt) stmt()          {}
 
 func (ng *Engine) newTestQuery(f func(context.Context) error) Query {
 	qry := &query{
@@ -327,8 +355,10 @@ func (ng *Engine) newTestQuery(f func(context.Context) error) Query {
 //
 // At this point per query only one EvalStmt is evaluated. Alert and record
 // statements are not handled by the Engine.
-func (ng *Engine) exec(q *query) (model.Value, error) {
-	ctx, cancel := context.WithTimeout(q.ng.baseCtx, ng.options.Timeout)
+func (ng *Engine) exec(ctx context.Context, q *query) (model.Value, error) {
+	currentQueries.Inc()
+	defer currentQueries.Dec()
+	ctx, cancel := context.WithTimeout(ctx, ng.options.Timeout)
 	q.cancel = cancel
 
 	queueTimer := q.stats.GetTimer(stats.ExecQueueTime).Start()
@@ -365,35 +395,19 @@ func (ng *Engine) exec(q *query) (model.Value, error) {
 
 // execEvalStmt evaluates the expression of an evaluation statement for the given time range.
 func (ng *Engine) execEvalStmt(ctx context.Context, query *query, s *EvalStmt) (model.Value, error) {
-	prepareTimer := query.stats.GetTimer(stats.TotalQueryPreparationTime).Start()
-	analyzeTimer := query.stats.GetTimer(stats.QueryAnalysisTime).Start()
-
-	// Only one execution statement per query is allowed.
-	analyzer := &Analyzer{
-		Storage: ng.storage,
-		Expr:    s.Expr,
-		Start:   s.Start,
-		End:     s.End,
-	}
-	err := analyzer.Analyze(ctx)
+	querier, err := ng.queryable.Querier()
 	if err != nil {
-		analyzeTimer.Stop()
-		prepareTimer.Stop()
 		return nil, err
 	}
-	analyzeTimer.Stop()
+	defer querier.Close()
 
-	preloadTimer := query.stats.GetTimer(stats.PreloadTime).Start()
-	closer, err := analyzer.Prepare(ctx)
-	if err != nil {
-		preloadTimer.Stop()
-		prepareTimer.Stop()
-		return nil, err
-	}
-	defer closer.Close()
-
-	preloadTimer.Stop()
+	prepareTimer := query.stats.GetTimer(stats.QueryPreparationTime).Start()
+	err = ng.populateIterators(ctx, querier, s)
 	prepareTimer.Stop()
+	if err != nil {
+		return nil, err
+	}
+	defer ng.closeIterators(s)
 
 	evalTimer := query.stats.GetTimer(stats.InnerEvalTime).Start()
 	// Instant evaluation.
@@ -498,8 +512,63 @@ func (ng *Engine) execEvalStmt(ctx context.Context, query *query, s *EvalStmt) (
 	return resMatrix, nil
 }
 
+func (ng *Engine) populateIterators(ctx context.Context, querier local.Querier, s *EvalStmt) error {
+	var queryErr error
+	Inspect(s.Expr, func(node Node) bool {
+		switch n := node.(type) {
+		case *VectorSelector:
+			if s.Start.Equal(s.End) {
+				n.iterators, queryErr = querier.QueryInstant(
+					ctx,
+					s.Start.Add(-n.Offset),
+					StalenessDelta,
+					n.LabelMatchers...,
+				)
+			} else {
+				n.iterators, queryErr = querier.QueryRange(
+					ctx,
+					s.Start.Add(-n.Offset-StalenessDelta),
+					s.End.Add(-n.Offset),
+					n.LabelMatchers...,
+				)
+			}
+			if queryErr != nil {
+				return false
+			}
+		case *MatrixSelector:
+			n.iterators, queryErr = querier.QueryRange(
+				ctx,
+				s.Start.Add(-n.Offset-n.Range),
+				s.End.Add(-n.Offset),
+				n.LabelMatchers...,
+			)
+			if queryErr != nil {
+				return false
+			}
+		}
+		return true
+	})
+	return queryErr
+}
+
+func (ng *Engine) closeIterators(s *EvalStmt) {
+	Inspect(s.Expr, func(node Node) bool {
+		switch n := node.(type) {
+		case *VectorSelector:
+			for _, it := range n.iterators {
+				it.Close()
+			}
+		case *MatrixSelector:
+			for _, it := range n.iterators {
+				it.Close()
+			}
+		}
+		return true
+	})
+}
+
 // An evaluator evaluates given expressions at a fixed timestamp. It is attached to an
-// engine through which it connects to a storage and reports errors. On timeout or
+// engine through which it connects to a querier and reports errors. On timeout or
 // cancellation of its context it terminates.
 type evaluator struct {
 	ctx context.Context
@@ -539,7 +608,7 @@ func (ev *evaluator) evalScalar(e Expr) *model.Scalar {
 	val := ev.eval(e)
 	sv, ok := val.(*model.Scalar)
 	if !ok {
-		ev.errorf("expected scalar but got %s", val.Type())
+		ev.errorf("expected scalar but got %s", documentedType(val.Type()))
 	}
 	return sv
 }
@@ -549,15 +618,18 @@ func (ev *evaluator) evalVector(e Expr) vector {
 	val := ev.eval(e)
 	vec, ok := val.(vector)
 	if !ok {
-		ev.errorf("expected vector but got %s", val.Type())
+		ev.errorf("expected instant vector but got %s", documentedType(val.Type()))
 	}
 	return vec
 }
 
 // evalInt attempts to evaluate e into an integer and errors otherwise.
-func (ev *evaluator) evalInt(e Expr) int {
+func (ev *evaluator) evalInt(e Expr) int64 {
 	sc := ev.evalScalar(e)
-	return int(sc.Value)
+	if !convertibleToInt64(sc.Value) {
+		ev.errorf("scalar value %v overflows int64", sc.Value)
+	}
+	return int64(sc.Value)
 }
 
 // evalFloat attempts to evaluate e into a float and errors otherwise.
@@ -567,11 +639,13 @@ func (ev *evaluator) evalFloat(e Expr) float64 {
 }
 
 // evalMatrix attempts to evaluate e into a matrix and errors otherwise.
+// The error message uses the term "range vector" to match the user facing
+// documentation.
 func (ev *evaluator) evalMatrix(e Expr) matrix {
 	val := ev.eval(e)
 	mat, ok := val.(matrix)
 	if !ok {
-		ev.errorf("expected matrix but got %s", val.Type())
+		ev.errorf("expected range vector but got %s", documentedType(val.Type()))
 	}
 	return mat
 }
@@ -581,7 +655,7 @@ func (ev *evaluator) evalString(e Expr) *model.String {
 	val := ev.eval(e)
 	sv, ok := val.(*model.String)
 	if !ok {
-		ev.errorf("expected string but got %s", val.Type())
+		ev.errorf("expected string but got %s", documentedType(val.Type()))
 	}
 	return sv
 }
@@ -590,7 +664,7 @@ func (ev *evaluator) evalString(e Expr) *model.String {
 func (ev *evaluator) evalOneOf(e Expr, t1, t2 model.ValueType) model.Value {
 	val := ev.eval(e)
 	if val.Type() != t1 && val.Type() != t2 {
-		ev.errorf("expected %s or %s but got %s", t1, t2, val.Type())
+		ev.errorf("expected %s or %s but got %s", documentedType(t1), documentedType(t2), documentedType(val.Type()))
 	}
 	return val
 }
@@ -681,14 +755,14 @@ func (ev *evaluator) eval(expr Expr) model.Value {
 // vectorSelector evaluates a *VectorSelector expression.
 func (ev *evaluator) vectorSelector(node *VectorSelector) vector {
 	vec := vector{}
-	for fp, it := range node.iterators {
+	for _, it := range node.iterators {
 		refTime := ev.Timestamp.Add(-node.Offset)
 		samplePair := it.ValueAtOrBeforeTime(refTime)
 		if samplePair.Timestamp.Before(refTime.Add(-StalenessDelta)) {
 			continue // Sample outside of staleness policy window.
 		}
 		vec = append(vec, &sample{
-			Metric:    node.metrics[fp],
+			Metric:    it.Metric(),
 			Value:     samplePair.Value,
 			Timestamp: ev.Timestamp,
 		})
@@ -704,7 +778,7 @@ func (ev *evaluator) matrixSelector(node *MatrixSelector) matrix {
 	}
 
 	sampleStreams := make([]*sampleStream, 0, len(node.iterators))
-	for fp, it := range node.iterators {
+	for _, it := range node.iterators {
 		samplePairs := it.RangeValues(interval)
 		if len(samplePairs) == 0 {
 			continue
@@ -717,7 +791,7 @@ func (ev *evaluator) matrixSelector(node *MatrixSelector) matrix {
 		}
 
 		sampleStream := &sampleStream{
-			Metric: node.metrics[fp],
+			Metric: it.Metric(),
 			Values: samplePairs,
 		}
 		sampleStreams = append(sampleStreams, sampleStream)
@@ -992,10 +1066,7 @@ func scalarBinop(op itemType, lhs, rhs model.SampleValue) model.SampleValue {
 	case itemPOW:
 		return model.SampleValue(math.Pow(float64(lhs), float64(rhs)))
 	case itemMOD:
-		if rhs != 0 {
-			return model.SampleValue(int(lhs) % int(rhs))
-		}
-		return model.SampleValue(math.NaN())
+		return model.SampleValue(math.Mod(float64(lhs), float64(rhs)))
 	case itemEQL:
 		return btos(lhs == rhs)
 	case itemNEQ:
@@ -1026,10 +1097,7 @@ func vectorElemBinop(op itemType, lhs, rhs model.SampleValue) (model.SampleValue
 	case itemPOW:
 		return model.SampleValue(math.Pow(float64(lhs), float64(rhs))), true
 	case itemMOD:
-		if rhs != 0 {
-			return model.SampleValue(int(lhs) % int(rhs)), true
-		}
-		return model.SampleValue(math.NaN()), true
+		return model.SampleValue(math.Mod(float64(lhs), float64(rhs))), true
 	case itemEQL:
 		return lhs, lhs == rhs
 	case itemNEQ:
@@ -1069,13 +1137,17 @@ type groupedAggregation struct {
 func (ev *evaluator) aggregation(op itemType, grouping model.LabelNames, without bool, keepCommon bool, param Expr, vec vector) vector {
 
 	result := map[uint64]*groupedAggregation{}
-	var k int
+	var k int64
 	if op == itemTopK || op == itemBottomK {
 		k = ev.evalInt(param)
 		if k < 1 {
 			return vector{}
 		}
 	}
+	var q float64
+	if op == itemQuantile {
+		q = ev.evalFloat(param)
+	}
 	var valueLabel model.LabelName
 	if op == itemCountValues {
 		valueLabel = model.LabelName(ev.evalString(param).Value)
@@ -1133,7 +1205,7 @@ func (ev *evaluator) aggregation(op itemType, grouping model.LabelNames, without
 				valuesSquaredSum: s.Value * s.Value,
 				groupCount:       1,
 			}
-			if op == itemTopK {
+			if op == itemTopK || op == itemQuantile {
 				result[groupingKey].heap = make(vectorByValueHeap, 0, k)
 				heap.Push(&result[groupingKey].heap, &sample{Value: s.Value, Metric: s.Metric})
 			} else if op == itemBottomK {
@@ -1168,19 +1240,21 @@ func (ev *evaluator) aggregation(op itemType, grouping model.LabelNames, without
 			groupedResult.valuesSquaredSum += s.Value * s.Value
 			groupedResult.groupCount++
 		case itemTopK:
-			if len(groupedResult.heap) < k || groupedResult.heap[0].Value < s.Value || math.IsNaN(float64(groupedResult.heap[0].Value)) {
-				if len(groupedResult.heap) == k {
+			if int64(len(groupedResult.heap)) < k || groupedResult.heap[0].Value < s.Value || math.IsNaN(float64(groupedResult.heap[0].Value)) {
+				if int64(len(groupedResult.heap)) == k {
 					heap.Pop(&groupedResult.heap)
 				}
 				heap.Push(&groupedResult.heap, &sample{Value: s.Value, Metric: s.Metric})
 			}
 		case itemBottomK:
-			if len(groupedResult.reverseHeap) < k || groupedResult.reverseHeap[0].Value > s.Value || math.IsNaN(float64(groupedResult.reverseHeap[0].Value)) {
-				if len(groupedResult.reverseHeap) == k {
+			if int64(len(groupedResult.reverseHeap)) < k || groupedResult.reverseHeap[0].Value > s.Value || math.IsNaN(float64(groupedResult.reverseHeap[0].Value)) {
+				if int64(len(groupedResult.reverseHeap)) == k {
 					heap.Pop(&groupedResult.reverseHeap)
 				}
 				heap.Push(&groupedResult.reverseHeap, &sample{Value: s.Value, Metric: s.Metric})
 			}
+		case itemQuantile:
+			groupedResult.heap = append(groupedResult.heap, s)
 		default:
 			panic(fmt.Errorf("expected aggregation operator but got %q", op))
 		}
@@ -1223,6 +1297,8 @@ func (ev *evaluator) aggregation(op itemType, grouping model.LabelNames, without
 				})
 			}
 			continue // Bypass default append.
+		case itemQuantile:
+			aggr.value = model.SampleValue(quantile(q, aggr.heap))
 		default:
 			// For other aggregations, we already have the right value.
 		}
@@ -1290,3 +1366,16 @@ func (g *queryGate) Done() {
 		panic("engine.queryGate.Done: more operations done than started")
 	}
 }
+
+// documentedType returns the internal type to the equivalent
+// user facing terminology as defined in the documentation.
+func documentedType(t model.ValueType) string {
+	switch t.String() {
+	case "vector":
+		return "instant vector"
+	case "matrix":
+		return "range vector"
+	default:
+		return t.String()
+	}
+}
