@@ -105,7 +105,7 @@ func TestAlertingRule(t *testing.T) {
 	for i, test := range tests {
 		evalTime := model.Time(0).Add(test.time)
 
-		res, err := rule.eval(evalTime, suite.QueryEngine(), "")
+		res, err := rule.Eval(suite.Context(), evalTime, suite.QueryEngine(), "")
 		if err != nil {
 			t.Fatalf("Error during alerting rule evaluation: %s", err)
 		}
@@ -136,6 +136,12 @@ func TestAlertingRule(t *testing.T) {
 			t.Errorf("%d. Expected and actual outputs don't match:", i)
 			t.Fatalf("Expected:\n%v\n----\nActual:\n%v", strings.Join(expected, "\n"), strings.Join(actual, "\n"))
 		}
+
+		for _, aa := range rule.ActiveAlerts() {
+			if _, ok := aa.Labels[model.MetricNameLabel]; ok {
+				t.Fatalf("%s label set on active alert: %s", model.MetricNameLabel, aa.Labels)
+			}
+		}
 	}
 }
 
