@@ -14,16 +14,18 @@
 package local
 
 import (
+	"errors"
 	"fmt"
 	"io"
 	"os"
-	"path"
+	"path/filepath"
 	"strings"
 	"sync/atomic"
 
 	"github.com/prometheus/common/log"
 	"github.com/prometheus/common/model"
 
+	"github.com/prometheus/prometheus/storage/local/chunk"
 	"github.com/prometheus/prometheus/storage/local/codable"
 	"github.com/prometheus/prometheus/storage/local/index"
 )
@@ -52,7 +54,7 @@ func (p *persistence) recoverFromCrash(fingerprintToSeries map[model.Fingerprint
 
 	log.Info("Scanning files.")
 	for i := 0; i < 1<<(seriesDirNameLen*4); i++ {
-		dirname := path.Join(p.basePath, fmt.Sprintf(seriesDirNameFmt, i))
+		dirname := filepath.Join(p.basePath, fmt.Sprintf(seriesDirNameFmt, i))
 		dir, err := os.Open(dirname)
 		if os.IsNotExist(err) {
 			continue
@@ -60,9 +62,9 @@ func (p *persistence) recoverFromCrash(fingerprintToSeries map[model.Fingerprint
 		if err != nil {
 			return err
 		}
-		defer dir.Close()
 		for fis := []os.FileInfo{}; err != io.EOF; fis, err = dir.Readdir(1024) {
 			if err != nil {
+				dir.Close()
 				return err
 			}
 			for _, fi := range fis {
@@ -76,6 +78,7 @@ func (p *persistence) recoverFromCrash(fingerprintToSeries map[model.Fingerprint
 				}
 			}
 		}
+		dir.Close()
 	}
 	log.Infof("File scan complete. %d series found.", len(fpsSeen))
 
@@ -113,10 +116,10 @@ func (p *persistence) recoverFromCrash(fingerprintToSeries map[model.Fingerprint
 					)
 				}
 				s.chunkDescs = append(
-					make([]*chunkDesc, 0, len(s.chunkDescs)-s.persistWatermark),
+					make([]*chunk.Desc, 0, len(s.chunkDescs)-s.persistWatermark),
 					s.chunkDescs[s.persistWatermark:]...,
 				)
-				numMemChunkDescs.Sub(float64(s.persistWatermark))
+				chunk.NumMemDescs.Sub(float64(s.persistWatermark))
 				s.persistWatermark = 0
 				s.chunkDescsOffset = 0
 			}
@@ -139,7 +142,13 @@ func (p *persistence) recoverFromCrash(fingerprintToSeries map[model.Fingerprint
 		}
 	}
 
-	p.setDirty(false)
+	p.dirtyMtx.Lock()
+	// Only declare storage clean if it didn't become dirty during crash recovery.
+	if !p.becameDirty {
+		p.dirty = false
+	}
+	p.dirtyMtx.Unlock()
+
 	log.Warn("Crash recovery complete.")
 	return nil
 }
@@ -175,36 +184,46 @@ func (p *persistence) sanitizeSeries(
 	fingerprintToSeries map[model.Fingerprint]*memorySeries,
 	fpm fpMappings,
 ) (model.Fingerprint, bool) {
-	filename := path.Join(dirname, fi.Name())
+	var (
+		fp       model.Fingerprint
+		err      error
+		filename = filepath.Join(dirname, fi.Name())
+		s        *memorySeries
+	)
+
 	purge := func() {
-		var err error
-		defer func() {
-			if err != nil {
-				log.Errorf("Failed to move lost series file %s to orphaned directory, deleting it instead. Error was: %s", filename, err)
-				if err = os.Remove(filename); err != nil {
-					log.Errorf("Even deleting file %s did not work: %s", filename, err)
-				}
+		if fp != 0 {
+			var metric model.Metric
+			if s != nil {
+				metric = s.metric
 			}
-		}()
-		orphanedDir := path.Join(p.basePath, "orphaned", path.Base(dirname))
-		if err = os.MkdirAll(orphanedDir, 0700); err != nil {
-			return
+			if err = p.quarantineSeriesFile(
+				fp, errors.New("purge during crash recovery"), metric,
+			); err == nil {
+				return
+			}
+			log.
+				With("file", filename).
+				With("error", err).
+				Error("Failed to move lost series file to orphaned directory.")
 		}
-		if err = os.Rename(filename, path.Join(orphanedDir, fi.Name())); err != nil {
-			return
+		// If we are here, we are either purging an incorrectly named
+		// file, or quarantining has failed. So simply delete the file.
+		if err = os.Remove(filename); err != nil {
+			log.
+				With("file", filename).
+				With("error", err).
+				Error("Failed to delete lost series file.")
 		}
 	}
 
-	var fp model.Fingerprint
-	var err error
-
 	if len(fi.Name()) != fpLen-seriesDirNameLen+len(seriesFileSuffix) ||
 		!strings.HasSuffix(fi.Name(), seriesFileSuffix) {
 		log.Warnf("Unexpected series file name %s.", filename)
 		purge()
 		return fp, false
 	}
-	if fp, err = model.FingerprintFromString(path.Base(dirname) + fi.Name()[:fpLen-seriesDirNameLen]); err != nil {
+	if fp, err = model.FingerprintFromString(filepath.Base(dirname) + fi.Name()[:fpLen-seriesDirNameLen]); err != nil {
 		log.Warnf("Error parsing file name %s: %s", filename, err)
 		purge()
 		return fp, false
@@ -273,8 +292,16 @@ func (p *persistence) sanitizeSeries(
 			)
 			s.chunkDescs = cds
 			s.chunkDescsOffset = 0
-			s.savedFirstTime = cds[0].firstTime()
-			s.lastTime = cds[len(cds)-1].lastTime()
+			s.savedFirstTime = cds[0].FirstTime()
+			s.lastTime, err = cds[len(cds)-1].LastTime()
+			if err != nil {
+				log.Errorf(
+					"Failed to determine time of the last sample for metric %v, fingerprint %v: %s",
+					s.metric, fp, err,
+				)
+				purge()
+				return fp, false
+			}
 			s.persistWatermark = len(cds)
 			s.modTime = modTime
 			return fp, true
@@ -289,7 +316,7 @@ func (p *persistence) sanitizeSeries(
 
 		// First, throw away the chunkDescs without chunks.
 		s.chunkDescs = s.chunkDescs[s.persistWatermark:]
-		numMemChunkDescs.Sub(float64(s.persistWatermark))
+		chunk.NumMemDescs.Sub(float64(s.persistWatermark))
 		cds, err := p.loadChunkDescs(fp, 0)
 		if err != nil {
 			log.Errorf(
@@ -301,13 +328,21 @@ func (p *persistence) sanitizeSeries(
 		}
 		s.persistWatermark = len(cds)
 		s.chunkDescsOffset = 0
-		s.savedFirstTime = cds[0].firstTime()
+		s.savedFirstTime = cds[0].FirstTime()
 		s.modTime = modTime
 
-		lastTime := cds[len(cds)-1].lastTime()
+		lastTime, err := cds[len(cds)-1].LastTime()
+		if err != nil {
+			log.Errorf(
+				"Failed to determine time of the last sample for metric %v, fingerprint %v: %s",
+				s.metric, fp, err,
+			)
+			purge()
+			return fp, false
+		}
 		keepIdx := -1
 		for i, cd := range s.chunkDescs {
-			if cd.firstTime() >= lastTime {
+			if cd.FirstTime() >= lastTime {
 				keepIdx = i
 				break
 			}
@@ -317,8 +352,8 @@ func (p *persistence) sanitizeSeries(
 				"Recovered metric %v, fingerprint %v: all %d chunks recovered from series file.",
 				s.metric, fp, chunksInFile,
 			)
-			numMemChunkDescs.Sub(float64(len(s.chunkDescs)))
-			atomic.AddInt64(&numMemChunks, int64(-len(s.chunkDescs)))
+			chunk.NumMemDescs.Sub(float64(len(s.chunkDescs)))
+			atomic.AddInt64(&chunk.NumMemChunks, int64(-len(s.chunkDescs)))
 			s.chunkDescs = cds
 			s.headChunkClosed = true
 			return fp, true
@@ -327,8 +362,12 @@ func (p *persistence) sanitizeSeries(
 			"Recovered metric %v, fingerprint %v: recovered %d chunks from series file, recovered %d chunks from checkpoint.",
 			s.metric, fp, chunksInFile, len(s.chunkDescs)-keepIdx,
 		)
-		numMemChunkDescs.Sub(float64(keepIdx))
-		atomic.AddInt64(&numMemChunks, int64(-keepIdx))
+		chunk.NumMemDescs.Sub(float64(keepIdx))
+		atomic.AddInt64(&chunk.NumMemChunks, int64(-keepIdx))
+		if keepIdx == len(s.chunkDescs) {
+			// No chunks from series file left, head chunk is evicted, so declare it closed.
+			s.headChunkClosed = true
+		}
 		s.chunkDescs = append(cds, s.chunkDescs[keepIdx:]...)
 		return fp, true
 	}
@@ -414,7 +453,10 @@ func (p *persistence) cleanUpArchiveIndexes(
 		if err != nil {
 			return err
 		}
-		series := newMemorySeries(model.Metric(m), cds, p.seriesFileModTime(model.Fingerprint(fp)))
+		series, err := newMemorySeries(model.Metric(m), cds, p.seriesFileModTime(model.Fingerprint(fp)))
+		if err != nil {
+			return err
+		}
 		fpToSeries[model.Fingerprint(fp)] = series
 		return nil
 	}); err != nil {
