@@ -25,6 +25,7 @@ import (
 
 	"github.com/prometheus/common/model"
 
+	"github.com/prometheus/prometheus/storage/local/chunk"
 	"github.com/prometheus/prometheus/storage/local/codable"
 	"github.com/prometheus/prometheus/storage/local/index"
 	"github.com/prometheus/prometheus/util/testutil"
@@ -38,10 +39,10 @@ var (
 	m5 = model.Metric{"label": "value5"}
 )
 
-func newTestPersistence(t *testing.T, encoding chunkEncoding) (*persistence, testutil.Closer) {
-	DefaultChunkEncoding = encoding
+func newTestPersistence(t *testing.T, encoding chunk.Encoding) (*persistence, testutil.Closer) {
+	chunk.DefaultEncoding = encoding
 	dir := testutil.NewTemporaryDirectory("test_persistence", t)
-	p, err := newPersistence(dir.Path(), false, false, func() bool { return false }, 0.1)
+	p, err := newPersistence(dir.Path(), false, false, func() bool { return false }, 0.15)
 	if err != nil {
 		dir.Close()
 		t.Fatal(err)
@@ -53,22 +54,22 @@ func newTestPersistence(t *testing.T, encoding chunkEncoding) (*persistence, tes
 	})
 }
 
-func buildTestChunks(t *testing.T, encoding chunkEncoding) map[model.Fingerprint][]chunk {
+func buildTestChunks(t *testing.T, encoding chunk.Encoding) map[model.Fingerprint][]chunk.Chunk {
 	fps := model.Fingerprints{
 		m1.FastFingerprint(),
 		m2.FastFingerprint(),
 		m3.FastFingerprint(),
 	}
-	fpToChunks := map[model.Fingerprint][]chunk{}
+	fpToChunks := map[model.Fingerprint][]chunk.Chunk{}
 
 	for _, fp := range fps {
-		fpToChunks[fp] = make([]chunk, 0, 10)
+		fpToChunks[fp] = make([]chunk.Chunk, 0, 10)
 		for i := 0; i < 10; i++ {
-			ch, err := newChunkForEncoding(encoding)
+			ch, err := chunk.NewForEncoding(encoding)
 			if err != nil {
 				t.Fatal(err)
 			}
-			chs, err := ch.add(model.SamplePair{
+			chs, err := ch.Add(model.SamplePair{
 				Timestamp: model.Time(i),
 				Value:     model.SampleValue(fp),
 			})
@@ -81,18 +82,18 @@ func buildTestChunks(t *testing.T, encoding chunkEncoding) map[model.Fingerprint
 	return fpToChunks
 }
 
-func chunksEqual(c1, c2 chunk) bool {
-	it1 := c1.newIterator()
-	it2 := c2.newIterator()
-	for it1.scan() && it2.scan() {
-		if !(it1.value() == it2.value()) {
+func chunksEqual(c1, c2 chunk.Chunk) bool {
+	it1 := c1.NewIterator()
+	it2 := c2.NewIterator()
+	for it1.Scan() && it2.Scan() {
+		if !(it1.Value() == it2.Value()) {
 			return false
 		}
 	}
-	return it1.err() == nil && it2.err() == nil
+	return it1.Err() == nil && it2.Err() == nil
 }
 
-func testPersistLoadDropChunks(t *testing.T, encoding chunkEncoding) {
+func testPersistLoadDropChunks(t *testing.T, encoding chunk.Encoding) {
 	p, closer := newTestPersistence(t, encoding)
 	defer closer.Close()
 
@@ -138,37 +139,61 @@ func testPersistLoadDropChunks(t *testing.T, encoding chunkEncoding) {
 			t.Errorf("Got %d chunkDescs, want %d.", len(actualChunkDescs), 10)
 		}
 		for i, cd := range actualChunkDescs {
-			lastTime, err := cd.lastTime()
+			lastTime, err := cd.LastTime()
 			if err != nil {
 				t.Fatal(err)
 			}
-			if cd.firstTime() != model.Time(i) || lastTime != model.Time(i) {
+			if cd.FirstTime() != model.Time(i) || lastTime != model.Time(i) {
 				t.Errorf(
 					"Want ts=%v, got firstTime=%v, lastTime=%v.",
-					i, cd.firstTime(), lastTime,
+					i, cd.FirstTime(), lastTime,
 				)
 			}
 
 		}
 		// Load chunk descs partially.
 		actualChunkDescs, err = p.loadChunkDescs(fp, 5)
+		if err != nil {
+			t.Fatal(err)
+		}
 		if len(actualChunkDescs) != 5 {
 			t.Errorf("Got %d chunkDescs, want %d.", len(actualChunkDescs), 5)
 		}
 		for i, cd := range actualChunkDescs {
-			lastTime, err := cd.lastTime()
+			lastTime, err := cd.LastTime()
 			if err != nil {
 				t.Fatal(err)
 			}
-			if cd.firstTime() != model.Time(i) || lastTime != model.Time(i) {
+			if cd.FirstTime() != model.Time(i) || lastTime != model.Time(i) {
 				t.Errorf(
 					"Want ts=%v, got firstTime=%v, lastTime=%v.",
-					i, cd.firstTime(), lastTime,
+					i, cd.FirstTime(), lastTime,
 				)
 			}
 
 		}
 	}
+	// Try to drop one chunk, which must be prevented by the shrink
+	// ratio. Since we do not pass in any chunks to persist, the offset
+	// should be the number of chunks in the file.
+	for fp, _ := range fpToChunks {
+		firstTime, offset, numDropped, allDropped, err := p.dropAndPersistChunks(fp, 1, nil)
+		if err != nil {
+			t.Fatal(err)
+		}
+		if offset != 10 {
+			t.Errorf("want offset 10, got %d", offset)
+		}
+		if firstTime != 0 {
+			t.Errorf("want first time 0, got %d", firstTime)
+		}
+		if numDropped != 0 {
+			t.Errorf("want 0 dropped chunks, got %v", numDropped)
+		}
+		if allDropped {
+			t.Error("all chunks dropped")
+		}
+	}
 	// Drop half of the chunks.
 	for fp, expectedChunks := range fpToChunks {
 		firstTime, offset, numDropped, allDropped, err := p.dropAndPersistChunks(fp, 5, nil)
@@ -399,14 +424,14 @@ func testPersistLoadDropChunks(t *testing.T, encoding chunkEncoding) {
 			t.Error("all chunks dropped")
 		}
 	}
-	// Drop only the first two chunks should not happen, either.
+	// Drop only the first two chunks should not happen, either. Chunks in file is now 9.
 	for fp := range fpToChunks {
 		firstTime, offset, numDropped, allDropped, err := p.dropAndPersistChunks(fp, 2, nil)
 		if err != nil {
 			t.Fatal(err)
 		}
-		if offset != 0 {
-			t.Errorf("want offset 0, got %d", offset)
+		if offset != 9 {
+			t.Errorf("want offset 9, got %d", offset)
 		}
 		if firstTime != 0 {
 			t.Errorf("want first time 0, got %d", firstTime)
@@ -447,7 +472,7 @@ func TestPersistLoadDropChunksType1(t *testing.T) {
 	testPersistLoadDropChunks(t, 1)
 }
 
-func testCheckpointAndLoadSeriesMapAndHeads(t *testing.T, encoding chunkEncoding) {
+func testCheckpointAndLoadSeriesMapAndHeads(t *testing.T, encoding chunk.Encoding) {
 	p, closer := newTestPersistence(t, encoding)
 	defer closer.Close()
 
@@ -461,7 +486,10 @@ func testCheckpointAndLoadSeriesMapAndHeads(t *testing.T, encoding chunkEncoding
 	s1.add(model.SamplePair{Timestamp: 1, Value: 3.14})
 	s3.add(model.SamplePair{Timestamp: 2, Value: 2.7})
 	s3.headChunkClosed = true
-	s3.persistWatermark = 1
+	// Create another chunk in s3.
+	s3.add(model.SamplePair{Timestamp: 3, Value: 1.4})
+	s3.headChunkClosed = true
+	s3.persistWatermark = 2
 	for i := 0; i < 10000; i++ {
 		s4.add(model.SamplePair{
 			Timestamp: model.Time(i),
@@ -496,7 +524,7 @@ func testCheckpointAndLoadSeriesMapAndHeads(t *testing.T, encoding chunkEncoding
 		if !reflect.DeepEqual(loadedS1.metric, m1) {
 			t.Errorf("want metric %v, got %v", m1, loadedS1.metric)
 		}
-		if !reflect.DeepEqual(loadedS1.head().c, s1.head().c) {
+		if !reflect.DeepEqual(loadedS1.head().C, s1.head().C) {
 			t.Error("head chunks differ")
 		}
 		if loadedS1.chunkDescsOffset != 0 {
@@ -505,11 +533,11 @@ func testCheckpointAndLoadSeriesMapAndHeads(t *testing.T, encoding chunkEncoding
 		if loadedS1.headChunkClosed {
 			t.Error("headChunkClosed is true")
 		}
-		if loadedS1.head().chunkFirstTime != 1 {
-			t.Errorf("want chunkFirstTime in head chunk to be 1, got %d", loadedS1.head().chunkFirstTime)
+		if loadedS1.head().ChunkFirstTime != 1 {
+			t.Errorf("want ChunkFirstTime in head chunk to be 1, got %d", loadedS1.head().ChunkFirstTime)
 		}
-		if loadedS1.head().chunkLastTime != model.Earliest {
-			t.Error("want chunkLastTime in head chunk to be unset")
+		if loadedS1.head().ChunkLastTime != model.Earliest {
+			t.Error("want ChunkLastTime in head chunk to be unset")
 		}
 	} else {
 		t.Errorf("couldn't find %v in loaded map", m1)
@@ -518,20 +546,20 @@ func testCheckpointAndLoadSeriesMapAndHeads(t *testing.T, encoding chunkEncoding
 		if !reflect.DeepEqual(loadedS3.metric, m3) {
 			t.Errorf("want metric %v, got %v", m3, loadedS3.metric)
 		}
-		if loadedS3.head().c != nil {
+		if loadedS3.head().C != nil {
 			t.Error("head chunk not evicted")
 		}
-		if loadedS3.chunkDescsOffset != 0 {
-			t.Errorf("want chunkDescsOffset 0, got %d", loadedS3.chunkDescsOffset)
+		if loadedS3.chunkDescsOffset != 1 {
+			t.Errorf("want chunkDescsOffset 1, got %d", loadedS3.chunkDescsOffset)
 		}
 		if !loadedS3.headChunkClosed {
 			t.Error("headChunkClosed is false")
 		}
-		if loadedS3.head().chunkFirstTime != 2 {
-			t.Errorf("want chunkFirstTime in head chunk to be 2, got %d", loadedS3.head().chunkFirstTime)
+		if loadedS3.head().ChunkFirstTime != 3 {
+			t.Errorf("want ChunkFirstTime in head chunk to be 3, got %d", loadedS3.head().ChunkFirstTime)
 		}
-		if loadedS3.head().chunkLastTime != 2 {
-			t.Errorf("want chunkLastTime in head chunk to be 2, got %d", loadedS3.head().chunkLastTime)
+		if loadedS3.head().ChunkLastTime != 3 {
+			t.Errorf("want ChunkLastTime in head chunk to be 3, got %d", loadedS3.head().ChunkLastTime)
 		}
 	} else {
 		t.Errorf("couldn't find %v in loaded map", m3)
@@ -546,10 +574,10 @@ func testCheckpointAndLoadSeriesMapAndHeads(t *testing.T, encoding chunkEncoding
 		if got, want := loadedS4.persistWatermark, 0; got != want {
 			t.Errorf("got persistWatermark %d, want %d", got, want)
 		}
-		if loadedS4.chunkDescs[2].isEvicted() {
+		if loadedS4.chunkDescs[2].IsEvicted() {
 			t.Error("3rd chunk evicted")
 		}
-		if loadedS4.chunkDescs[3].isEvicted() {
+		if loadedS4.chunkDescs[3].IsEvicted() {
 			t.Error("4th chunk evicted")
 		}
 		if loadedS4.chunkDescsOffset != 0 {
@@ -559,27 +587,27 @@ func testCheckpointAndLoadSeriesMapAndHeads(t *testing.T, encoding chunkEncoding
 			t.Error("headChunkClosed is true")
 		}
 		for i, cd := range loadedS4.chunkDescs {
-			if cd.chunkFirstTime != cd.c.firstTime() {
+			if cd.ChunkFirstTime != cd.C.FirstTime() {
 				t.Errorf(
-					"chunkDesc[%d]: chunkFirstTime not consistent with chunk, want %d, got %d",
-					i, cd.c.firstTime(), cd.chunkFirstTime,
+					"chunk.Desc[%d]: ChunkFirstTime not consistent with chunk, want %d, got %d",
+					i, cd.C.FirstTime(), cd.ChunkFirstTime,
 				)
 			}
 			if i == len(loadedS4.chunkDescs)-1 {
 				// Head chunk.
-				if cd.chunkLastTime != model.Earliest {
-					t.Error("want chunkLastTime in head chunk to be unset")
+				if cd.ChunkLastTime != model.Earliest {
+					t.Error("want ChunkLastTime in head chunk to be unset")
 				}
 				continue
 			}
-			lastTime, err := cd.c.newIterator().lastTimestamp()
+			lastTime, err := cd.C.NewIterator().LastTimestamp()
 			if err != nil {
 				t.Fatal(err)
 			}
-			if cd.chunkLastTime != lastTime {
+			if cd.ChunkLastTime != lastTime {
 				t.Errorf(
-					"chunkDesc[%d]: chunkLastTime not consistent with chunk, want %d, got %d",
-					i, lastTime, cd.chunkLastTime,
+					"chunk.Desc[%d]: ChunkLastTime not consistent with chunk, want %d, got %d",
+					i, lastTime, cd.ChunkLastTime,
 				)
 			}
 		}
@@ -590,20 +618,17 @@ func testCheckpointAndLoadSeriesMapAndHeads(t *testing.T, encoding chunkEncoding
 		if !reflect.DeepEqual(loadedS5.metric, m5) {
 			t.Errorf("want metric %v, got %v", m5, loadedS5.metric)
 		}
-		if got, want := len(loadedS5.chunkDescs), chunkCountS5; got != want {
+		if got, want := len(loadedS5.chunkDescs), chunkCountS5-3; got != want {
 			t.Errorf("got %d chunkDescs, want %d", got, want)
 		}
-		if got, want := loadedS5.persistWatermark, 3; got != want {
+		if got, want := loadedS5.persistWatermark, 0; got != want {
 			t.Errorf("got persistWatermark %d, want %d", got, want)
 		}
-		if !loadedS5.chunkDescs[2].isEvicted() {
-			t.Error("3rd chunk not evicted")
+		if loadedS5.chunkDescs[0].IsEvicted() {
+			t.Error("1st chunk evicted")
 		}
-		if loadedS5.chunkDescs[3].isEvicted() {
-			t.Error("4th chunk evicted")
-		}
-		if loadedS5.chunkDescsOffset != 0 {
-			t.Errorf("want chunkDescsOffset 0, got %d", loadedS5.chunkDescsOffset)
+		if loadedS5.chunkDescsOffset != 3 {
+			t.Errorf("want chunkDescsOffset 3, got %d", loadedS5.chunkDescsOffset)
 		}
 		if loadedS5.headChunkClosed {
 			t.Error("headChunkClosed is true")
@@ -611,32 +636,32 @@ func testCheckpointAndLoadSeriesMapAndHeads(t *testing.T, encoding chunkEncoding
 		for i, cd := range loadedS5.chunkDescs {
 			if i < 3 {
 				// Evicted chunks.
-				if cd.chunkFirstTime == model.Earliest {
-					t.Errorf("chunkDesc[%d]: chunkLastTime not set", i)
+				if cd.ChunkFirstTime == model.Earliest {
+					t.Errorf("chunk.Desc[%d]: ChunkLastTime not set", i)
 				}
 				continue
 			}
-			if cd.chunkFirstTime != cd.c.firstTime() {
+			if cd.ChunkFirstTime != cd.C.FirstTime() {
 				t.Errorf(
-					"chunkDesc[%d]: chunkFirstTime not consistent with chunk, want %d, got %d",
-					i, cd.c.firstTime(), cd.chunkFirstTime,
+					"chunk.Desc[%d]: ChunkFirstTime not consistent with chunk, want %d, got %d",
+					i, cd.C.FirstTime(), cd.ChunkFirstTime,
 				)
 			}
 			if i == len(loadedS5.chunkDescs)-1 {
 				// Head chunk.
-				if cd.chunkLastTime != model.Earliest {
-					t.Error("want chunkLastTime in head chunk to be unset")
+				if cd.ChunkLastTime != model.Earliest {
+					t.Error("want ChunkLastTime in head chunk to be unset")
 				}
 				continue
 			}
-			lastTime, err := cd.c.newIterator().lastTimestamp()
+			lastTime, err := cd.C.NewIterator().LastTimestamp()
 			if err != nil {
 				t.Fatal(err)
 			}
-			if cd.chunkLastTime != lastTime {
+			if cd.ChunkLastTime != lastTime {
 				t.Errorf(
-					"chunkDesc[%d]: chunkLastTime not consistent with chunk, want %d, got %d",
-					i, cd.chunkLastTime, lastTime,
+					"chunk.Desc[%d]: ChunkLastTime not consistent with chunk, want %d, got %d",
+					i, cd.ChunkLastTime, lastTime,
 				)
 			}
 		}
@@ -687,7 +712,7 @@ func TestCheckpointAndLoadFPMappings(t *testing.T) {
 	}
 }
 
-func testFingerprintsModifiedBefore(t *testing.T, encoding chunkEncoding) {
+func testFingerprintsModifiedBefore(t *testing.T, encoding chunk.Encoding) {
 	p, closer := newTestPersistence(t, encoding)
 	defer closer.Close()
 
@@ -766,7 +791,7 @@ func TestFingerprintsModifiedBeforeChunkType2(t *testing.T) {
 	testFingerprintsModifiedBefore(t, 2)
 }
 
-func testDropArchivedMetric(t *testing.T, encoding chunkEncoding) {
+func testDropArchivedMetric(t *testing.T, encoding chunk.Encoding) {
 	p, closer := newTestPersistence(t, encoding)
 	defer closer.Close()
 
@@ -840,7 +865,7 @@ type incrementalBatch struct {
 	expectedLpToFps index.LabelPairFingerprintsMapping
 }
 
-func testIndexing(t *testing.T, encoding chunkEncoding) {
+func testIndexing(t *testing.T, encoding chunk.Encoding) {
 	batches := []incrementalBatch{
 		{
 			fpToMetric: index.FingerprintMetricMapping{
@@ -1045,7 +1070,10 @@ func verifyIndexedState(i int, t *testing.T, b incrementalBatch, indexedFpsToMet
 
 	// Compare label name -> label values mappings.
 	for ln, lvs := range b.expectedLnToLvs {
-		outLvs := p.labelValuesForLabelName(ln)
+		outLvs, err := p.labelValuesForLabelName(ln)
+		if err != nil {
+			t.Fatal(err)
+		}
 
 		outSet := codable.LabelValueSet{}
 		for _, lv := range outLvs {
@@ -1090,11 +1118,12 @@ func TestQuranatineSeriesFile(t *testing.T) {
 			t.Errorf("Unexpected state of quarantined file %q. Expected it to exist: %t. os.Stat returned: %s.", quarantinedFile, seriesFileShouldExist, err)
 		}
 		f, err := os.Open(hintFile)
-		defer f.Close()
 		if err != nil {
 			t.Errorf("Could not open hint file %q: %s", hintFile, err)
 			return
 		}
+		defer f.Close()
+
 		scanner := bufio.NewScanner(f)
 		for _, want := range contentHintFile {
 			if !scanner.Scan() {
