@@ -28,6 +28,7 @@ import (
 	"golang.org/x/net/context"
 
 	"github.com/prometheus/prometheus/promql"
+	"github.com/prometheus/prometheus/retrieval"
 	"github.com/prometheus/prometheus/storage/local"
 	"github.com/prometheus/prometheus/storage/metric"
 	"github.com/prometheus/prometheus/util/httputil"
@@ -66,6 +67,14 @@ func (e *apiError) Error() string {
 	return fmt.Sprintf("%s: %s", e.typ, e.err)
 }
 
+type targetRetriever interface {
+	Targets() []*retrieval.Target
+}
+
+type alertmanagerRetriever interface {
+	Alertmanagers() []string
+}
+
 type response struct {
 	Status    status      `json:"status"`
 	Data      interface{} `json:"data,omitempty"`
@@ -88,17 +97,22 @@ type API struct {
 	Storage     local.Storage
 	QueryEngine *promql.Engine
 
+	targetRetriever       targetRetriever
+	alertmanagerRetriever alertmanagerRetriever
+
 	context func(r *http.Request) context.Context
 	now     func() model.Time
 }
 
 // NewAPI returns an initialized API type.
-func NewAPI(qe *promql.Engine, st local.Storage) *API {
+func NewAPI(qe *promql.Engine, st local.Storage, tr targetRetriever, ar alertmanagerRetriever) *API {
 	return &API{
-		QueryEngine: qe,
-		Storage:     st,
-		context:     route.Context,
-		now:         model.Now,
+		QueryEngine:           qe,
+		Storage:               st,
+		targetRetriever:       tr,
+		alertmanagerRetriever: ar,
+		context:               route.Context,
+		now:                   model.Now,
 	}
 }
 
@@ -129,6 +143,9 @@ func (api *API) Register(r *route.Router) {
 
 	r.Get("/series", instr("series", api.series))
 	r.Del("/series", instr("drop_series", api.dropSeries))
+
+	r.Get("/targets", instr("targets", api.targets))
+	r.Get("/alertmanagers", instr("alertmanagers", api.alertmanagers))
 }
 
 type queryData struct {
@@ -157,7 +174,7 @@ func (api *API) query(r *http.Request) (interface{}, *apiError) {
 		return nil, &apiError{errorBadData, err}
 	}
 
-	res := qry.Exec()
+	res := qry.Exec(api.context(r))
 	if res.Err != nil {
 		switch res.Err.(type) {
 		case promql.ErrQueryCanceled:
@@ -182,11 +199,21 @@ func (api *API) queryRange(r *http.Request) (interface{}, *apiError) {
 	if err != nil {
 		return nil, &apiError{errorBadData, err}
 	}
+	if end.Before(start) {
+		err := errors.New("end timestamp must not be before start time")
+		return nil, &apiError{errorBadData, err}
+	}
+
 	step, err := parseDuration(r.FormValue("step"))
 	if err != nil {
 		return nil, &apiError{errorBadData, err}
 	}
 
+	if step <= 0 {
+		err := errors.New("zero or negative query resolution step widths are not accepted. Try a positive integer")
+		return nil, &apiError{errorBadData, err}
+	}
+
 	// For safety, limit the number of returned points per timeseries.
 	// This is sufficient for 60s resolution for a week or 1h resolution for a year.
 	if end.Sub(start)/step > 11000 {
@@ -199,7 +226,7 @@ func (api *API) queryRange(r *http.Request) (interface{}, *apiError) {
 		return nil, &apiError{errorBadData, err}
 	}
 
-	res := qry.Exec()
+	res := qry.Exec(api.context(r))
 	if res.Err != nil {
 		switch res.Err.(type) {
 		case promql.ErrQueryCanceled:
@@ -221,7 +248,16 @@ func (api *API) labelValues(r *http.Request) (interface{}, *apiError) {
 	if !model.LabelNameRE.MatchString(name) {
 		return nil, &apiError{errorBadData, fmt.Errorf("invalid label name: %q", name)}
 	}
-	vals := api.Storage.LabelValuesForLabelName(model.LabelName(name))
+	q, err := api.Storage.Querier()
+	if err != nil {
+		return nil, &apiError{errorExec, err}
+	}
+	defer q.Close()
+
+	vals, err := q.LabelValuesForLabelName(api.context(r), model.LabelName(name))
+	if err != nil {
+		return nil, &apiError{errorExec, err}
+	}
 	sort.Sort(vals)
 
 	return vals, nil
@@ -255,19 +291,24 @@ func (api *API) series(r *http.Request) (interface{}, *apiError) {
 		end = model.Latest
 	}
 
-	res := map[model.Fingerprint]metric.Metric{}
-
-	for _, lm := range r.Form["match[]"] {
-		matchers, err := promql.ParseMetricSelector(lm)
+	var matcherSets []metric.LabelMatchers
+	for _, s := range r.Form["match[]"] {
+		matchers, err := promql.ParseMetricSelector(s)
 		if err != nil {
 			return nil, &apiError{errorBadData, err}
 		}
-		for fp, met := range api.Storage.MetricsForLabelMatchers(
-			start, end,
-			matchers...,
-		) {
-			res[fp] = met
-		}
+		matcherSets = append(matcherSets, matchers)
+	}
+
+	q, err := api.Storage.Querier()
+	if err != nil {
+		return nil, &apiError{errorExec, err}
+	}
+	defer q.Close()
+
+	res, err := q.MetricsForLabelMatchers(api.context(r), start, end, matcherSets...)
+	if err != nil {
+		return nil, &apiError{errorExec, err}
 	}
 
 	metrics := make([]model.Metric, 0, len(res))
@@ -282,32 +323,88 @@ func (api *API) dropSeries(r *http.Request) (interface{}, *apiError) {
 	if len(r.Form["match[]"]) == 0 {
 		return nil, &apiError{errorBadData, fmt.Errorf("no match[] parameter provided")}
 	}
-	fps := map[model.Fingerprint]struct{}{}
 
-	for _, lm := range r.Form["match[]"] {
-		matchers, err := promql.ParseMetricSelector(lm)
+	numDeleted := 0
+	for _, s := range r.Form["match[]"] {
+		matchers, err := promql.ParseMetricSelector(s)
 		if err != nil {
 			return nil, &apiError{errorBadData, err}
 		}
-		for fp := range api.Storage.MetricsForLabelMatchers(
-			model.Earliest, model.Latest, // Get every series.
-			matchers...,
-		) {
-			fps[fp] = struct{}{}
+		n, err := api.Storage.DropMetricsForLabelMatchers(context.TODO(), matchers...)
+		if err != nil {
+			return nil, &apiError{errorExec, err}
 		}
-	}
-	for fp := range fps {
-		api.Storage.DropMetricsForFingerprints(fp)
+		numDeleted += n
 	}
 
 	res := struct {
 		NumDeleted int `json:"numDeleted"`
 	}{
-		NumDeleted: len(fps),
+		NumDeleted: numDeleted,
 	}
 	return res, nil
 }
 
+type Target struct {
+	// Labels before any processing.
+	DiscoveredLabels model.LabelSet `json:"discoveredLabels"`
+	// Any labels that are added to this target and its metrics.
+	Labels model.LabelSet `json:"labels"`
+
+	ScrapeURL string `json:"scrapeUrl"`
+
+	LastError  string                 `json:"lastError"`
+	LastScrape time.Time              `json:"lastScrape"`
+	Health     retrieval.TargetHealth `json:"health"`
+}
+
+type TargetDiscovery struct {
+	ActiveTargets []*Target `json:"activeTargets"`
+}
+
+func (api *API) targets(r *http.Request) (interface{}, *apiError) {
+	targets := api.targetRetriever.Targets()
+	res := &TargetDiscovery{ActiveTargets: make([]*Target, len(targets))}
+
+	for i, t := range targets {
+		lastErrStr := ""
+		lastErr := t.LastError()
+		if lastErr != nil {
+			lastErrStr = lastErr.Error()
+		}
+
+		res.ActiveTargets[i] = &Target{
+			DiscoveredLabels: t.DiscoveredLabels(),
+			Labels:           t.Labels(),
+			ScrapeURL:        t.URL().String(),
+			LastError:        lastErrStr,
+			LastScrape:       t.LastScrape(),
+			Health:           t.Health(),
+		}
+	}
+
+	return res, nil
+}
+
+type AlertmanagerDiscovery struct {
+	ActiveAlertmanagers []*AlertmanagerTarget `json:"activeAlertmanagers"`
+}
+
+type AlertmanagerTarget struct {
+	URL string `json:"url"`
+}
+
+func (api *API) alertmanagers(r *http.Request) (interface{}, *apiError) {
+	urls := api.alertmanagerRetriever.Alertmanagers()
+	ams := &AlertmanagerDiscovery{ActiveAlertmanagers: make([]*AlertmanagerTarget, len(urls))}
+
+	for i := range urls {
+		ams.ActiveAlertmanagers[i] = &AlertmanagerTarget{URL: urls[i]}
+	}
+
+	return ams, nil
+}
+
 func respond(w http.ResponseWriter, data interface{}) {
 	w.Header().Set("Content-Type", "application/json")
 	w.WriteHeader(http.StatusOK)
